// AppView.cpp : implementation of the CAppScrollView class
//

#include "stdafx.h"
#include "BigScrl.h"

#include "MainFrm.h"

#include "BigDoc.h"
#include "BigView.h"
#include "AppView.h"

#include <limits.h>

#define ZOOM_VIEW 0

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// Define some colors to make the boxes better looking
#define NUM_COLORS 9
#define COLOR_YELLOW  RGB(255, 255,   0)
#define COLOR_PURPLE  RGB(255,   0, 255)
#define COLOR_BLACK   RGB(  0,   0,   0)
#define COLOR_RED     RGB(255,   0,   0)
#define COLOR_GREEN   RGB(  0, 255,   0)
#define COLOR_DKGREEN RGB(  0, 128,   0)
#define COLOR_BLUE    RGB(  0,   0, 255)
#define COLOR_AQUA    RGB(  0, 255, 255)
#define COLOR_GRAY    RGB(192, 192, 192)
COLORREF colors[NUM_COLORS] = {COLOR_BLACK, COLOR_RED, COLOR_GREEN, COLOR_DKGREEN, 
	COLOR_BLUE, COLOR_AQUA, COLOR_YELLOW, COLOR_PURPLE, COLOR_GRAY};

// This sample is build with MM_LOMETRIC. You can use all maping mode
// in CBigScrollView, but your derived class must reflect your mode.

///////////////////////////////////////////////////////////////////////
// Clipping region

// To draw a line correctly, it is necessary to clip it on the client
// area. This insure that the GDI coordinates are in the range of
// -32768 <-> 32767.

CClipRegion::CClipRegion()
{
	memset(ClipRect, 0, MaxReg * sizeof(long));
}

CClipRegion::CClipRegion(CRect &rectClip)
{
	ClipRect[right]  = rectClip.right;
	ClipRect[left]   = rectClip.left;
	ClipRect[top]    = rectClip.top;
	ClipRect[bottom] = rectClip.bottom;
}

int CClipRegion::ClipRegion(long* p)
{
	int Reg = 0;
	if		(*p >= ClipRect[right])  Reg |= RegRight;
	else if (*p <= ClipRect[left])   Reg |= RegLeft;
	p++;
	if		(*p >= ClipRect[top])    Reg |= RegTop;
	else if (*p <= ClipRect[bottom]) Reg |= RegBottom;
	return Reg;
}

void CClipRegion::Intersect(long *r, long *a, long *b, short Plane)
{
	static short c[MaxReg][2] = { x,y, x,y, y,x, y,x };
	short c1 = c[Plane][x];
	short c2 = c[Plane][y];
	long mul, div, r1, r2;

	mul = ClipRect[Plane] - a[c1];
	div = b[c1] - a[c1];
	r1 = a[c1] + MulDiv(b[c1] - a[c1], mul, div);
	r2 = a[c2] + MulDiv(b[c2] - a[c2], mul, div);
	r[c1] = r1; r[c2] = r2;
}

BOOL CClipRegion::PtInRect(CPoint &p)
{
	if (p.x >= ClipRect[right])  return FALSE;
	if (p.x <  ClipRect[left])   return FALSE;
	if (p.y >  ClipRect[top])    return FALSE;
	if (p.y <= ClipRect[bottom]) return FALSE;
	return TRUE;
}

BOOL CClipRegion::ClipLine(CPoint &p1, CPoint &p2)
{
	short Reg1, Reg2;
	register short i;
	static short ClipReg[MaxReg] =	{ RegRight, RegLeft, RegTop, RegBottom };

	long *p = &p1.x, *q = &p2.x;

	Reg1 = ClipRegion(p);
	Reg2 = ClipRegion(q);

	if (Reg1 | Reg2)
	{
		for (i = 0; i < MaxReg; i++)
			if ((Reg1 == Reg2) && (Reg2 == ClipReg[i])) return FALSE;

		for (i = MaxReg - 1; i >= 0; i--)
		{
			if (Reg1 & Reg2 & ClipReg[i]) return FALSE;
			if ((Reg1 | Reg2) & ClipReg[i])
			{
				if (Reg1 & ClipReg[i])
					Intersect(p, q, p, i);
				else
					Intersect(q, p, q, i);
				if (i >= 0)
				{
					Reg1 = ClipRegion(p);
					Reg2 = ClipRegion(q);
				}
			}
		}
	}
	return TRUE;
}

BOOL CClipRegion::IsRectOnRect(CRect& rect)
{
  if (rect.left   >= ClipRect[right])  return FALSE;
  if (rect.right  <= ClipRect[left])   return FALSE;
  if (rect.top    <= ClipRect[bottom]) return FALSE;
  if (rect.bottom >= ClipRect[top])    return FALSE;
  return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CAppScrollView

const int MapMode = MM_LOMETRIC;

IMPLEMENT_DYNCREATE(CAppScrollView, CBigScrollView)

BEGIN_MESSAGE_MAP(CAppScrollView, CBigScrollView)
	//{{AFX_MSG_MAP(CAppScrollView)
	ON_COMMAND(ID_VIEW_ZOOM_IN, OnViewZoomIn)
	ON_COMMAND(ID_VIEW_ZOOM_OUT, OnViewZoomOut)
	ON_COMMAND(ID_VIEW_ZOOM_DEFAULT, OnViewZoomDefault)
	ON_WM_MOUSEMOVE()
	ON_WM_HSCROLL()
	ON_WM_VSCROLL()
	ON_WM_SIZE()
	ON_COMMAND(ID_VIEW_SET_SCALE_TO_FIT, OnViewSetScaleToFit)
	ON_COMMAND(ID_VIEW_RESIZE_PARENT_TO_FIT, OnViewResizeParentToFit)
	ON_COMMAND(ID_VIEW_UPPER_LEFT, OnViewUpperLeft)
	ON_WM_LBUTTONDOWN()
	ON_WM_SETCURSOR()
	ON_COMMAND(ID_TRACK_MODE, OnTrackMode)
	ON_UPDATE_COMMAND_UI(ID_TRACK_MODE, OnUpdateTrackMode)
	ON_WM_RBUTTONDOWN()
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CBigScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CBigScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CBigScrollView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CAppScrollView construction/destruction

CAppScrollView::CAppScrollView()
{
	m_sizeDoc = CSize(2100, 2960);	// default to A4, MM_LOMETRIC
	m_Scale = m_Zoom = ZOOM_ONE;
	m_ZoomMin = 1;
	m_ZoomMax = LONG_MAX / 2960;
	m_x0 = 2100 / 2;
	m_y0 = 2960 / 2;
	m_hCursor = AfxGetApp()->LoadCursor(IDC_HANDCURSOR);
	m_TrackMode = 1;
	m_fontHeight = 0;
}

CAppScrollView::~CAppScrollView()
{
	if (m_Font.m_hObject != NULL) m_Font.DeleteObject();
	if (m_hCursor != NULL) DestroyCursor(m_hCursor);
}

BOOL CAppScrollView::PreCreateWindow(CREATESTRUCT& cs)
{
	return CBigScrollView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CAppScrollView drawing

void CAppScrollView::OnPrepareDC(CDC* pDC, CPrintInfo* pInfo) 
{
	CRect rect;
	CBigScrollView::OnPrepareDC(pDC, pInfo);
	if (pDC->IsPrinting())
	{
		int cx = 10 * pDC->GetDeviceCaps(HORZSIZE);
		int cy = 10 * pDC->GetDeviceCaps(VERTSIZE);
		rect.SetRect(0, 0, cx, -cy);

		m_xo = 0;							// no offset for printing
		m_yo = cy;
		m_Scale = ZOOM_ONE;					// no zoom for printing
	}
	else
	{
		GetLogClientRect(&rect);
		rect.OffsetRect(-m_Offset);			// clip rect in [lu]
		//rect.OffsetRect(0, m_Delta);		// correct window

		m_xo = m_Offset.x;					// offset for drawing
		m_yo = m_Offset.y + m_TotalSize.cy;	// origin bottom/left
		m_Scale = m_Zoom;					// zoom for drawing
	}
	rect.InflateRect(m_Delta, -m_Delta);
	m_clipReg = CClipRegion(rect);
	UpdateGDIObjects();
}

void CAppScrollView::OnDraw(CDC* pDC)
{
#if ZOOM_VIEW
	CPen *pOldPen;
	CFont *pOldFont;
	char buf[12];

	long xl = MulDiv(m_sizeDoc.cx, 95, 100);	// 95%
	long yl = MulDiv(m_sizeDoc.cy, 95, 100);	// 95%
	
	long dx = xl / 10;
	long dy = yl / 10;

	long cx = dx / 10;
	long cy = dy / 10;

	long x1 = - 5 * dx;
	long y1 = - 5 * dy;
	
	// Draw border
	CRect rect(x1, y1, x1 + xl, y1 + yl);
	DrawClipRect(pDC, rect);

	// Draw a 10x10 grid of boxes of boxs - alternating color

	int color = 0;
	pOldFont = pDC->SelectObject(&m_Font);
	pDC->SetBkMode(TRANSPARENT);
	for (int row = 0; row < 10; row++)
	{
		for (int col = 0; col < 10; col++)
		{
			CPen newPen(PS_SOLID, 0, colors[color]); // 0 width = 1 device unit
			pOldPen = pDC->SelectObject(&newPen);
			rect.SetRect(x1 + col * dx + cx, y1 + row * dy + dy - cy,
						 x1 + col * dx + dx - cx, y1 + row * dy + cy);
			DrawClipRect(pDC, rect);
			pDC->SelectObject(pOldPen);
			
			// Rotate colors
			color = (color + 1) % NUM_COLORS;

			// Draw the row / col text
			wsprintf(buf, "row %d", row);
			SIZE size; GetTextExtentPoint32(pDC->m_hDC, buf, strlen(buf), &size);
			// in client units
			size.cx = MulDiv(size.cx, ZOOM_ONE, m_Scale);
			size.cy = MulDiv(size.cy, ZOOM_ONE, m_Scale);
			long x = rect.left + (rect.Width() - size.cx) / 2;
			long y = rect.bottom - rect.Height() / 2 + size.cy;
			DrawClipText(pDC, x, y, CString(buf));

			y -= size.cy;
			wsprintf(buf, "col %d", col);
			DrawClipText(pDC, x, y, CString(buf));
		}
	}
	pDC->SelectObject(pOldFont);
#else
	int i;

	CBigScrollDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	long xl = MulDiv(m_sizeDoc.cx, 95, 100);	// 95%
	long yl = MulDiv(m_sizeDoc.cy, 95, 100);	// 95%

	// to avoid rounding problem, use dx & dy
	long dx = xl / 20;
	long dy = yl / 20;

	long x1 = - 10 * dx;
	long y1 = - 10 * dy;

	long x2 = x1 + 20 * dx;
	long y2 = y1 + 20 * dy;

	CPen red(PS_SOLID, 1, RGB(255,0,0));
	CPen green(PS_SOLID, 1, RGB(0,255,0));
	CPen blue(PS_SOLID, 1, RGB(0,0,255));

	CPen *pOldPen;
	pOldPen = pDC->SelectObject(&red);
	for (i = 0; i <= 20; i++)
	{
		long xt = x1 + i * dx;
		long xb = x1 + (20 - i) * dx;
		DrawClipLine(pDC, xt, y1, xb, y2);
	}
	for (i = 1; i < 20; i++)
	{
		long yt = y1 + i * dy;
		long yb = y1 + (20 - i) * dy;
		DrawClipLine(pDC, x1, yt, x2, yb);
	}

	pDC->SelectObject(&green);
	for (i = 0; i <= 20; i++)
	{
		long x = x1 + i * dx;
		DrawClipLine(pDC, x, y1, x, y2);
	}

	pDC->SelectObject(&blue);
	for (i = 0; i <= 20; i++)
	{
		long y = y1 + i * dy;
		DrawClipLine(pDC, x1, y, x2, y);
	}

	// select font and draw text if possible
	ASSERT(m_Font.m_hObject != NULL);
	pDC->SetBkMode(TRANSPARENT);
	CFont *pOldFont = pDC->SelectObject(&m_Font);
	DrawClipText(pDC, 50, 50, CString("BigScrollView"));
	pDC->SelectObject(pOldFont);

	pDC->SelectObject(pOldPen);
	blue.DeleteObject();
	green.DeleteObject();
	red.DeleteObject();
#endif
}

void CAppScrollView::UpdateGDIObjects()
{
	// calculate new font height
	int Height = m_sizeDoc.cx / 50;
	Height = MulDiv(Height, m_Scale, ZOOM_ONE);

	// if font valid and same font, return
	if (m_Font.m_hObject != NULL && Height == m_fontHeight) return;

	// delete old font
	if (m_Font.m_hObject != NULL)
		m_Font.DeleteObject();

	// create font
	LOGFONT lf;	memset(&lf, 0, sizeof(lf));
	lf.lfHeight = Height;
	lf.lfWeight = FW_NORMAL;
	lf.lfPitchAndFamily = DEFAULT_PITCH;
	strcpy(lf.lfFaceName, "Arial");
	VERIFY(m_Font.CreateFontIndirect(&lf));

	// save new font height
	m_fontHeight = Height;
}

long CAppScrollView::xPos(int x)
{
	return MulDiv(x + m_x0, m_Scale, ZOOM_ONE) - m_xo;
}

long CAppScrollView::yPos(int y)
{
	return MulDiv(y + m_y0, m_Scale, ZOOM_ONE) - m_yo;
}

int CAppScrollView::PosX(long x)
{
	return MulDiv(x, ZOOM_ONE, m_Scale) - m_x0;
}

int CAppScrollView::PosY(long y)
{
	return MulDiv(y, ZOOM_ONE, m_Scale) + m_y0;
}

void CAppScrollView::DrawClipLine(CDC *pDC, long x1, long y1, long x2, long y2)
{
	CPoint p1(xPos(x1), yPos(y1));
	CPoint p2(xPos(x2), yPos(y2));
	if (m_clipReg.ClipLine(p1, p2))
	{
		pDC->MoveTo(p1.x, p1.y);
		pDC->LineTo(p2.x, p2.y);
	}
}

void CAppScrollView::DrawClipRect(CDC *pDC, const CRect &rect)
{
	CPoint p1, p2;

	p1 = CPoint(xPos(rect.left), yPos(rect.top));
	p2 = CPoint(xPos(rect.right), yPos(rect.top));
	if (m_clipReg.ClipLine(p1, p2))
	{
		pDC->MoveTo(p1.x, p1.y);
		pDC->LineTo(p2.x, p2.y);
	}

	p1 = CPoint(xPos(rect.right), yPos(rect.top));
	p2 = CPoint(xPos(rect.right), yPos(rect.bottom));
	if (m_clipReg.ClipLine(p1, p2))
	{
		pDC->MoveTo(p1.x, p1.y);
		pDC->LineTo(p2.x, p2.y);
	}

	p1 = CPoint(xPos(rect.right), yPos(rect.bottom));
	p2 = CPoint(xPos(rect.left), yPos(rect.bottom));
	if (m_clipReg.ClipLine(p1, p2))
	{
		pDC->MoveTo(p1.x, p1.y);
		pDC->LineTo(p2.x, p2.y);
	}

	p1 = CPoint(xPos(rect.left), yPos(rect.bottom));
	p2 = CPoint(xPos(rect.left), yPos(rect.top));
	if (m_clipReg.ClipLine(p1, p2))
	{
		pDC->MoveTo(p1.x, p1.y);
		pDC->LineTo(p2.x, p2.y);
	}
}

void CAppScrollView::DrawClipText(CDC *pDC, long x, long y, CString &strText)
{
	// If a character has points outside the correct scroll size range
	// it is not draw with the GDI function !
	
	// this routine is for horizontal text only !
	
	// start position
	x = xPos(x), y = yPos(y);
	CRect rect(x, y, x, y);
	
	// check all characters
	LPCSTR p = strText, s = NULL;
	int len = strText.GetLength();
	for (int i = 0; i < len; i++, p++)
	{
		SIZE sizeChar;
		GetTextExtentPoint32(pDC->m_hDC, p, 1, &sizeChar);
		rect.right += sizeChar.cx;
		if (i == 0) rect.bottom -= sizeChar.cy;
		
		if (-32768 <= rect.left && rect.left <= 32767)
			if (m_clipReg.IsRectOnRect(rect))
				pDC->TextOut(rect.left, rect.top, p, 1);
		
		rect.left = rect.right;	
		if (rect.left > 32767) break;
	}	
}

void CAppScrollView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
	CBigScrollDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	m_sizeDoc = pDoc->GetDocSize();
	m_x0 = m_sizeDoc.cx / 2;
	m_y0 = m_sizeDoc.cy / 2;
	SetScrollSizes(MapMode, m_sizeDoc);
	OnZoom(ZOOM_ONE);
}

/////////////////////////////////////////////////////////////////////////////
// CAppScrollView printing

BOOL CAppScrollView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CAppScrollView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
}

void CAppScrollView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
}

/////////////////////////////////////////////////////////////////////////////
// CAppScrollView diagnostics

#ifdef _DEBUG
void CAppScrollView::AssertValid() const
{
	CBigScrollView::AssertValid();
}

void CAppScrollView::Dump(CDumpContext& dc) const
{
	CBigScrollView::Dump(dc);
}

CBigScrollDoc* CAppScrollView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CBigScrollDoc)));
	return (CBigScrollDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CAppScrollView helpers

void CAppScrollView::UpdateDisplayInfo() const
{
	CMainFrame *pMF = (CMainFrame*)AfxGetMainWnd();
	pMF->m_xs = m_Offset.x;
	pMF->m_ys = m_Offset.y;
}

/////////////////////////////////////////////////////////////////////////////
// CAppScrollView message handlers

void CAppScrollView::OnZoom(long Zoom)
{
	if (Zoom < m_ZoomMin)
	{
		MessageBeep(MB_ICONEXCLAMATION);
		Zoom = m_ZoomMin;
	}
	if (Zoom > m_ZoomMax)
	{
		MessageBeep(MB_ICONEXCLAMATION);
		Zoom = m_ZoomMax;
	}
	if (Zoom != m_Zoom)
	{
		m_Zoom = Zoom;
		CSize sizeTotal = m_sizeDoc;
		sizeTotal.cx = MulDiv(sizeTotal.cx, m_Zoom, ZOOM_ONE);
		sizeTotal.cy = MulDiv(sizeTotal.cy, m_Zoom, ZOOM_ONE);
		SetScrollSizes(MapMode, sizeTotal);
		UpdateDisplayInfo();
	}
}

void CAppScrollView::OnViewZoomIn() 
{
	OnZoom(m_Zoom * 2);
}

void CAppScrollView::OnViewZoomOut() 
{
	OnZoom(m_Zoom / 2);
}

void CAppScrollView::OnViewZoomDefault() 
{
	// get client center in document units
	CRect rectClient;
	GetLogClientRect(&rectClient);
	CPoint pointCenter = rectClient.CenterPoint();
	pointCenter.x = MulDiv(pointCenter.x, ZOOM_ONE, m_Zoom);
	pointCenter.y = MulDiv(pointCenter.y, ZOOM_ONE, m_Zoom);
	
	// change zoom
	OnZoom(ZOOM_ONE);

	// calculate new center point with zoom
	pointCenter.x = MulDiv(pointCenter.x, m_Zoom, ZOOM_ONE);
	pointCenter.y = MulDiv(pointCenter.y, m_Zoom, ZOOM_ONE);

	// center on new point
	CenterOnPoint(pointCenter);
}

void CAppScrollView::OnMouseMove(UINT nFlags, CPoint point) 
{
	CMainFrame *pMF = (CMainFrame*)AfxGetMainWnd();
	CPoint devPoint = GetLogPosition(point);
	pMF->m_xv = devPoint.x; pMF->m_yv = devPoint.y;
	devPoint.x = PosX(devPoint.x);
	devPoint.y = PosY(devPoint.y);
	pMF->m_xl = devPoint.x; pMF->m_yl = devPoint.y;
	CBigScrollView::OnMouseMove(nFlags, point);
	UpdateDisplayInfo();
}

void CAppScrollView::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	// only to update display infos
	CBigScrollView::OnHScroll(nSBCode, nPos, pScrollBar);
	UpdateDisplayInfo();
}

void CAppScrollView::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	// only to update display infos
	CBigScrollView::OnVScroll(nSBCode, nPos, pScrollBar);
	UpdateDisplayInfo();
}

void CAppScrollView::OnSize(UINT nType, int cx, int cy) 
{
	// only to update display infos
	CBigScrollView::OnSize(nType, cx, cy);
	UpdateDisplayInfo();
}

void CAppScrollView::OnViewSetScaleToFit()
{
	CSize sizeTotal = m_TotalSize;
	// this function change the scale
	SetScaleToFitSize(m_TotalSize);
	// read the scale back
	OnZoom(MulDiv(m_Zoom, m_TotalSize.cx, sizeTotal.cx));
}

void CAppScrollView::OnViewResizeParentToFit() 
{
	ResizeParentToFit(TRUE);
}

void CAppScrollView::OnViewUpperLeft() 
{
	ScrollToPosition(CPoint(0, 0));
}

class CHandTracker : public CRectTracker
{
public:
	CAppScrollView *m_pView;
	CHandTracker(CAppScrollView *pView);
	virtual void DrawTrackerRect(LPCRECT lpRect, CWnd* pWndClipTo, CDC* pDC, CWnd* pWnd);
	virtual void OnChangedRect(const CRect& rectOld);
};

CHandTracker::CHandTracker(CAppScrollView *pView)
{
	ASSERT(pView != NULL);
	m_pView = pView;
	m_sizeMin = CSize(1, 1);
}

void CHandTracker::DrawTrackerRect(LPCRECT lpRect, CWnd* pWndClipTo, CDC* pDC, CWnd* pWnd)
{
	// draw nothing
}

void CHandTracker::OnChangedRect(const CRect& rectOld)
{
	CPoint point;
	// call the view to update scroll position
	point.x = m_rect.Width() - rectOld.Width();
	point.y = m_rect.Height() - rectOld.Height();
	m_pView->UpdateScrollPosition(point);
}

void CAppScrollView::UpdateScrollPosition(CPoint point)
{
	CPoint pointCenter = GetLogClientCenter();
	if (m_nMapMode != MM_TEXT)
	{
		CWindowDC dc(NULL);
		dc.SetMapMode(m_nMapMode);
		dc.DPtoLP(&point);
	}
	pointCenter -= point;
	CenterOnPoint(pointCenter);
}

void CAppScrollView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	if (m_TrackMode == 1)
	{
		CHandTracker trackHand(this);
		CPoint saveCenter = m_Center;
		trackHand.TrackRubberBand(this, point);
		Invalidate();
		return;
	}
	else
	{
		CRectTracker tracker;
		if (tracker.TrackRubberBand(this, point)) 
		{
			CRect rectTracker = tracker.m_rect;
			rectTracker.NormalizeRect();
			if (rectTracker.Width() >= 8 * m_Delta)
			{
				long Zoom = m_Zoom;
				long saveZoom = m_Zoom;
								
				// get logical position from device position
				CPoint p1 = GetLogPosition(rectTracker.TopLeft());
				CPoint p2 = GetLogPosition(rectTracker.BottomRight());

				// get size of tracker rectangle
				rectTracker = CRect(p1, p2);
				CSize sizeTracker(abs(rectTracker.Width()), abs(rectTracker.Height()));
			
				// get center of tracker rectangle without zoom
				CPoint pointCenter = rectTracker.CenterPoint();
				pointCenter.x = MulDiv(pointCenter.x, ZOOM_ONE, m_Zoom);
				pointCenter.y = MulDiv(pointCenter.y, ZOOM_ONE, m_Zoom);
			
				// get client size
				CRect rectClient;
				GetLogClientRect(&rectClient);
				CSize sizeClient(rectClient.Width(), abs(rectClient.Height()));

				// calculate new zoom
				if (sizeClient.cy < MulDiv(sizeTracker.cy, sizeClient.cx, sizeTracker.cx))
					Zoom = MulDiv(Zoom, sizeClient.cy, sizeTracker.cy);
				else
					Zoom = MulDiv(Zoom, sizeClient.cx, sizeTracker.cx);

				// change zoom
				OnZoom(Zoom);

				// calculate new center point with new zoom
				if (m_Zoom != saveZoom)
				{
					pointCenter.x = MulDiv(pointCenter.x, m_Zoom, ZOOM_ONE);
					pointCenter.y = MulDiv(pointCenter.y, m_Zoom, ZOOM_ONE);
					// center on new point
					CenterOnPoint(pointCenter);
				}
			}
			return;
		}
	}
	CBigScrollView::OnLButtonDown(nFlags, point);
}

BOOL CAppScrollView::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message) 
{
	if (m_TrackMode == 1 && nHitTest == HTCLIENT)
	{
		::SetCursor(m_hCursor);
		return 0;
	}
	return CBigScrollView::OnSetCursor(pWnd, nHitTest, message);
}

void CAppScrollView::OnTrackMode() 
{
	m_TrackMode = (m_TrackMode == 0) ? 1: 0;
}

void CAppScrollView::OnUpdateTrackMode(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_TrackMode == 1);
}

void CAppScrollView::OnRButtonDown(UINT nFlags, CPoint point) 
{
	OnTrackMode();
	CBigScrollView::OnRButtonDown(nFlags, point);
}
