// AppView.h : interface of the CAppScrollView class
//
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// interface of the CClipRegion class

class CClipRegion
{
public:
	enum { x, y };
	enum { RegLeft = 0x01, RegRight = 0x02, RegTop = 0x04, RegBottom = 0x08 };
	enum { right, left, top, bottom };
	enum { MaxReg = 4 };
	CClipRegion();
	CClipRegion(CRect &rectClip);
	BOOL ClipLine(CPoint &p1, CPoint &p2);
	BOOL PtInRect(CPoint &p);
	BOOL IsRectOnRect(CRect& rect);
private:
	long ClipRect[MaxReg];
	int  ClipRegion(long *p);
	void Intersect(long *r, long *a, long *b, short Plane);
};

/////////////////////////////////////////////////////////////////////////////
// interface of the the CAppScrollView class

// [vlu] = virtual logical unit = logical unit + offset

#define ZOOM_ONE 1000

class CAppScrollView : public CBigScrollView
{
protected: // create from serialization only
	CAppScrollView();
	DECLARE_DYNCREATE(CAppScrollView)

// Attributes
public:
	CBigScrollDoc* GetDocument();
	CSize m_sizeDoc;	// document size
	long  m_x0, m_y0;	// document origin

	void  OnZoom(long Zoom);
	long  m_Zoom, m_ZoomMin, m_ZoomMax;
	
	long  m_Scale;		// zoom for drawing function
	long  m_xo, m_yo;	// origine for drawing function

// Operations
public:
	// GDI can only draw into valid logical units
	// so it is necessary to clip drawing object into valid range

	// convert client units into logical units (without offset)
	long xPos(int x);
	long yPos(int y);
	
	// convert virtual logical units into client units
	int PosX(long x);
	int PosY(long y);
	
	CClipRegion m_clipReg;		// clipping region
	void DrawClipLine(CDC *pDC, long x1, long y1, long x2, long y2);
	void DrawClipRect(CDC *pDC, const CRect &rect);
	void DrawClipText(CDC *pDC, long x, long y, CString &strText);

	void UpdateGDIObjects();
	void UpdateDisplayInfo() const;
	void UpdateScrollPosition(CPoint point);

	CFont m_Font;
	long m_fontHeight;
	HCURSOR m_hCursor;
	int m_TrackMode;

	// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAppScrollView)
	public:
	virtual void OnDraw(CDC* pDC);  // overridden to draw this view
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	virtual void OnPrepareDC(CDC* pDC, CPrintInfo* pInfo = NULL);
	protected:
	virtual BOOL OnPreparePrinting(CPrintInfo* pInfo);
	virtual void OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnEndPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint);
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CAppScrollView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

// Generated message map functions
protected:
	//{{AFX_MSG(CAppScrollView)
	afx_msg void OnViewZoomIn();
	afx_msg void OnViewZoomOut();
	afx_msg void OnViewZoomDefault();
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg void OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnViewSetScaleToFit();
	afx_msg void OnViewResizeParentToFit();
	afx_msg void OnViewUpperLeft();
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg BOOL OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message);
	afx_msg void OnTrackMode();
	afx_msg void OnUpdateTrackMode(CCmdUI* pCmdUI);
	afx_msg void OnRButtonDown(UINT nFlags, CPoint point);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

#ifndef _DEBUG  // debug version in BigView.cpp
inline CBigScrollDoc* CAppScrollView::GetDocument()
   { return (CBigScrollDoc*)m_pDocument; }
#endif

/////////////////////////////////////////////////////////////////////////////
